% Function to reproduce the figures in the paper "Dimensionality-
% reduced subspace clustering" by Reinhard Heckel, Michael Tschannen, and
% Helmut Boelcskei, 2015
%
% mode: 1: Face clustering (Sec. 5.2)
%       2: Gene expression data (Sec. 5.4)
%       3: Synthetic data (Sec. 5.1.1)
%       4: Handwritten digits (Sec. 5.3)
%       5: Synthetic data (Sec. 5.1.3)
% frac: Fraction of intersecting dimensions in the synthetic data experiment
%       (mode = 3) (optional, preferably integer multiple of 0.05)


function randProjSC(mode, frac)

    addpath ./include/SSC_ADMM_v1.1/
    addpath ./include/TSC_OMP/
    
    s = RandStream('mcg16807','Seed',1);
    RandStream.setGlobalStream(s);

    % initialization
    switch mode
        
        % face clustering
        case 1
            
            if ~exist('./include/faces/Xo.mat','file')
                addpath ./include/faces/
                read_yale_images;
                movefile('./Xo.mat','./include/faces');
            end
            
            load('./include/faces/Xo.mat','Xo');

            nrep = 100;
            prep = 5;
            L = 2;
            
            alpha = 5;
            q = 2;
            smax = 4;
            
            ps = ceil(10.^(0.6:0.08:3.4));

            selfaces = zeros(L,nrep);

            for n = 1:nrep
                selfaces(:,n) = randsample(length(Xo),L);
            end
            
            singvals = [];
            for i = 1:length(Xo)
                nimg = size(Xo{i},2);
                Xl = reshape(Xo{i},192,nimg*168);
                Xl = Xl(1:4:end,1:4:end);
                Xl = reshape(Xl,48*42,nimg);
                [~,S,~] = svd(Xl);
                sv = diag(S);
                singvals = [singvals sv(1:59)];
            end

            suffix = 'Faces';

        % gene expression data
        case 2
            addpath ./include/cancerdata/

            nrep = 1;
            prep = 200;
            L = 4;
            
            alpha = 25;
            q = 20;
            smax = 10;
            
            ps = ceil(10.^(0.6:0.08:2.69));
            
            A = tabread('Novartis_BPLC.top1000.gct');
            X = A(:,3:105);
            labels = dlmread('Novartis_BPLC.cls',' ',[2 0 2 102]) + 1;

            X = normc(X);
            
            suffix = 'Cancer';
            
            singvals = [];
            for i = 1:L
                [~,S,~] = svd(X(:,labels == i));
                sv = diag(S);
                singvals = [singvals sv(1:15)];
            end

        % synthetic data
        case 3
            m = 32768;
            d = 20;
            L = 3;
            nl = 80;
            nrep = 5;
            prep = 5;
            
            if nargin < 2
                frac = 0.4;
            end
            
            alpha = 1000;
            q = 16;
            smax = 6;
            
            ps = ceil(10.^(0.6:0.08:3.4));
            suffix = strcat('Synth',num2str(frac));
            
        % handwritten digits
        case 4
            digidx = [3,5,9];
            L = 3;
            nrep = 100;
            nl = 250;
            prep = 5;
            
            alpha = 5;
            q = 6;
            smax = 8;
            
            ps = ceil(10.^(0.6:0.08:2.8));
            
            addpath ./include/digits/
            images = loadMNISTImages('t10k-images-idx3-ubyte');
            labels = loadMNISTLabels('t10k-labels-idx1-ubyte');

            [labelssorted,idx] = sort(labels);
            imgssorted = normc(images(:,idx));
            
            beg(1) = 1;
            k = 1;
            beg(k) = 1; 
            for i =1:size(images,2) % for each point
                if labelssorted(i) == k-1;
                else
                    endd(k) = i-1;
                    k = k+1;
                    beg(k) = i;
                end
            end
            endd(k) = size(images,2);
            
            singvals = [];
            for l = 1:L
                [~,S,~] = svd(imgssorted(:,beg(digidx(l)):endd(digidx(l))));
                sv = diag(S);
                singvals = [singvals sv(1:100)];
            end
            
            suffix = 'Digits';
            
        
        % dimensionality reduction when the subspaces span the ambient space
        case 5
            m = 200;
            d = 20;
            L = 10;
            nl = 60;
            nrep = 5;
            prep = 5;
            
            alpha = 1000;
            q = 14;
            smax = 8;
            
            ps = ceil(10.^(0.4:0.15:2.2));
            suffix = 'SynthFR';
    end
    
    % save data matrix singular values
    if mode < 3 || mode == 4
        dlmwrite(strcat('SingularValues',suffix,'.dat'),[(1:size(singvals,1))' singvals],...
            'delimiter',' ');
    end

    cegtot = zeros(3,length(ps));
    ceftot = zeros(3,length(ps));
    tcprojtot = zeros(2,length(ps));
    tcsctot = zeros(3,length(ps));
    
    numerrs = 0;
    numprobinst = 0;

    for n = 1:nrep

        % generate data set instances
        switch mode
            case 1
                X = [];
                labels = [];
                for l = 1:L
                    X = [X Xo{selfaces(l,n)}];
                    labels = [labels ones(1, size( Xo{selfaces(l,n)} , 2))*l];
                end

                X = normc(double(X));

            case 3
                X = [];
                labels = [];
                UC = orth(randn(m,d*frac));
                for l=1:L
                    % orthonormal basis U consisting of the common part UC
                    % and the individual part with 1-frac entries
                    U = orth([UC,randn(m,d*(1-frac))] );
                    A = normc(randn(d,nl));
                    X = [X U*A];
                    labels = [labels ones(1,nl)*l];
                end
                
            case 4
                X = [];
                labels = [];
                for l = 1:L
                    curridx = beg(digidx(l)):endd(digidx(l));
                    selidx = curridx(randsample(length(curridx),nl));
                    X = [X imgssorted(:,selidx)];
                    labels = [labels ones(1,nl)*l];
                end
                X = normc(X);
                
             case 5
                X = [];
                labels = [];
                for l=1:L
                    U = orth(randn(m,d));
                    A = normc(randn(d,nl));
                    X = [X U*A];
                    labels = [labels ones(1,nl)*l];
                end
        end
        
        % test algorithms
        for p = 1:prep
            try
                [ceg, cef, tcproj, tcsc] = testSCAlgos(X,labels,ps,alpha,q,smax);

                cegtot = cegtot + ceg;
                ceftot = ceftot + cef;
                tcprojtot = tcprojtot + tcproj;
                tcsctot = tcsctot + tcsc;
                
                numprobinst = numprobinst + 1;
            catch
                numerrs = numerrs + 1;
            end
        end
    end
    
    cegtot = cegtot/numprobinst;
    ceftot = ceftot/numprobinst;
    tcprojtot = tcprojtot/numprobinst;
    tcsctot = tcsctot/numprobinst;
    
    fprintf('There were %i error(s) during this simulation \n',numerrs);
    
    % save results
    dlmwrite(strcat('TComp',suffix,'.dat'),[ps' ...
            tcprojtot' tcsctot'],'delimiter',' ');
    dlmwrite(strcat('CE',suffix,'.dat'),[ps' ...
        cegtot' ceftot'],'delimiter',' ');
end