% Function to test the clustering performance and running time of SSC,
% SSC-OMP, and TSC for dimensionality-reduced data (via Gaussian and fast
% random projection)
%
% X: m x N matrix of N data points
% labels: labels of the data points
% ps: vector of embedding dimensions to be tested
% alpha: SSC parameter
% q: TSC parameter
% smax: SSC-OMP parameter
%
% ceg: clustering errors for Gaussian random projection
% cef: clustering errors for fast random projection
% tcproj: running times of projections
% tcsc: running times of clustering algorithms

function [ceg, cef, tcproj, tcsc] = testSCAlgos(X,labels,ps,alpha,q,smax)
    nps = length(ps);
    m = size(X,1);
    N = size(X,2);
    ceg = zeros(3,nps);
    cef = zeros(3,nps);
    tcproj = zeros(2,nps);
    tcsc = zeros(3,nps);
    
    affine = false; rho = 1;
    
    for p = 1:nps
        
        % Gaussian random projection
        tic
        Xg = 1/sqrt(ps(p))*randn(ps(p),m)*X;
        tcproj(1,p) = toc;
        
        % fast random projection
        tic
        d = 2*binornd(ones(1,m),0.5)'-1;
        Xf = zeros(m,N);
        for n = 1:N
            Xf(:,n) = real(fft(d.*X(:,n)))';
        end
        Xf = Xf(randsample(m,ps(p)),:);
        tcproj(2,p) = toc; 
        
        
        % test algorithms
        % (running times are averaged for the two dimensionality reduction
        % methods)
        tic
        Csscg = admmLasso_mat_func(Xg,affine,alpha);
        Csscsymg = BuildAdjacency(thrC(Csscg,rho));
        gg = SpectralClustering(Csscsymg,max(labels));
        tcsc(1,p) = tcsc(1,p) + toc/2;
        cesscg = Misclassification(gg,labels);
        ceg(1,p) = ceg(1,p) + cesscg;
        
        tic
        labelstscg = TSCb(Xg,q,max(labels));
        tcsc(2,p) = tcsc(2,p) + toc/2;
        cetscg = Misclassification(labelstscg,labels);
        ceg(2,p) = ceg(2,p) + cetscg;
        
        tic
        labelsompg = OMPSSC(Xg,smax,max(labels));
        tcsc(3,p) = tcsc(3,p) + toc/2;
        ceompg = Misclassification(labelsompg,labels);
        ceg(3,p) = ceg(3,p) + ceompg;
        
        tic
        Csscf = admmLasso_mat_func(Xf,affine,alpha);
        Csscsymf = BuildAdjacency(thrC(Csscf,rho));
        gf = SpectralClustering(Csscsymf,max(labels));
        tcsc(1,p) = tcsc(1,p) + toc/2;
        cesscf = Misclassification(gf,labels);
        cef(1,p) = cef(1,p) + cesscf;
        
        tic
        labelstscf = TSCb(Xf,q,max(labels));
        tcsc(2,p) = tcsc(2,p) + toc/2;
        cetscf = Misclassification(labelstscf,labels);
        cef(2,p) = cef(2,p) + cetscf;
        
        tic
        labelsompf = OMPSSC(Xf,smax,max(labels));
        tcsc(3,p) = tcsc(3,p) + toc/2;
        ceompf = Misclassification(labelsompf,labels);
        cef(3,p) = cef(3,p) + ceompf;
    end
    
end